__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

import os
from django.http import HttpResponse

from .models import AmisRenderList
from django.conf import settings
import platform
from django.urls import get_urlconf
from django.shortcuts import render

g_sys_name = platform.system()


def no_route_found(request):
    nof = HttpResponse()
    nof.content = 'no page found:'+request.path
    nof.status_code = 404
    url_confs = get_urlconf()
    print(url_confs)

    return nof

def load_default_file(request):
    return no_route_found(request)


def render_this_one(request, re_one):

    file_url = re_one.json_file_url
    page_name = re_one.html_template
    print('render_amis_json', file_url)
    if (page_name is None) or len(page_name) == 0:
        page_name = 'html/index_for_json.html'
    if len(file_url)>3:
        if file_url[-4:]=='html':
            page_name = file_url
    return render(request, page_name, {'json_to_render_file_url': file_url})


def jhtml_render(request):
    path = request.path
    print(path)

    re_all = AmisRenderList.objects.all()
    for i in re_all:
        if i.page_url.strip() == path.strip():
            return render_this_one(request, i)

    #其他文件
    return load_default_file(request)


__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

from django.http import HttpResponse
from django.conf import settings
import os
from .models import AmisRenderList
import pathlib
from django.urls import reverse


def if_file_is_amis_json(file_path):
    if file_path[-4:]!='json':
        return False
    if 'amis_json' not in file_path:
        return False
    if 'static' not in file_path:
        return False
    return True


def add_one_file_to_table(file_path):
    curr = AmisRenderList.objects.filter(file_path=file_path).all()
    if len(curr)>0:
        #already exist, just skip
        return
    curr = AmisRenderList()
    curr.file_path = file_path
    curr.file_type = 'json'

    front_path = reverse('django_amis_render_default_list')
    if front_path[-1]=='/':
        front_path=front_path[:-1]

    pos = front_path.rfind('/')
    front_path=front_path[:pos]
    curr.page_url=front_path+'/jhtml/'+pathlib.Path(file_path).name[:-5]

    pos = file_path.find('static')

    curr.json_file_url ='/'+file_path[pos:]
    curr.json_file_url = curr.json_file_url.replace('\\','/')



    curr.html_template = None
    curr.save()

def auto_add(request):
    bdir = settings.BASE_DIR
    #找到目录下，所有目录名称为amis_json的目录，下面的 json文件，都添加路由
    amis_json_file_list = []
    print('auto_add bdir:', bdir)
    for root, dirs, files in os.walk(bdir, topdown=False):
        #print(root)
        blen = len(str(bdir))
        if str(bdir) == root[0:blen]:
            root=root[blen:]
        for name in files:
            file_name =os.path.join(root, name)
            print(file_name)
            if if_file_is_amis_json(file_name):
                amis_json_file_list.append(file_name)

    print(amis_json_file_list)
    for i in amis_json_file_list:
        add_one_file_to_table(i)

    return HttpResponse('ok')


__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"


from django.contrib import admin
from django.urls import path, re_path
from django.urls import path, include
from django.views.generic.base import RedirectView

from django.urls import re_path, path, include
from .default_list import default_list
from .auto_add import auto_add
from .jhtml_render import jhtml_render
from .amis_update import update_amis_local_to_editor, update_amis_editor_to_local
urlpatterns = [
    path('auto_add/', auto_add),
    path('default_list/', default_list, name='django_amis_render_default_list'),
    path('update_amis_local_to_editor/', update_amis_local_to_editor),
    path('update_amis_editor_to_local/', update_amis_editor_to_local),

    re_path('^jhtml/.*$', jhtml_render),
]




#https://books.agiliq.com/projects/django-admin-cookbook/en/latest/action_buttons.html

from django.contrib import admin
from .models import AmisRenderList
from django.utils.html import format_html
from django.urls import path
from django.http import HttpResponse,HttpResponseRedirect

# Register your models here.

from .auto_add import auto_add

#@admin.action(description='自动添加json文件和路径')
#def make_published(modeladmin, request, queryset):
#    auto_add(request)

class AmisRenderListAdmin(admin.ModelAdmin):

    change_list_template = "html/auto_add_list.html"

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path('auto_add_action/', self.auto_add_action),
        ]
        return my_urls + urls


    def auto_add_action(self, request):
        auto_add(request)
        self.message_user(request, "OK")
        return HttpResponseRedirect("../")

    def button_link(self, obj):
        button_html = """<a class="changelink" href="%s">打开页面</a>"""%(obj.page_url)
        return format_html(button_html)

    button_link.short_description = "打开"

    def button_link_edit(self, obj):
        button_html = """<a class="changelink" href="%s">编辑页面</a>"""%(obj.page_url)
        return format_html(button_html)

    button_link_edit.short_description = "编辑"

    list_display = ['id', 'page_url', 'file_path', 'button_link']
    #actions = [make_published]

    class Media:
        js = ('django_amis_render/jquery-3.6.0.min.js',)# 'django_amis_render/add_button.js')

admin.site.register(AmisRenderList, AmisRenderListAdmin)



from django.db import models


# Create your models here.

class AmisRenderList(models.Model):
    page_url = models.TextField(blank=True, null=True, help_text='')
    file_path = models.TextField(blank=True, null=True, help_text='')
    html_template = models.TextField(blank=True, null=True, help_text='')
    json_file_url = models.TextField(blank=True, null=True, help_text='')
    file_type = models.TextField(blank=True, null=True, help_text='supported file type: json or html')

    class Meta:
        managed = True
        verbose_name_plural = 'AMIS-PAGE'
        db_table = 'amis_render_list'



__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"




def default_list(request):
    #return HttpResponse()
    return 'ok'


# coding:utf-8

__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

from django.http import HttpResponse



"""
{"pages":[{"id":"1","icon":"fa fa-file","path":"hello-world","label":"Hello world","schema":{"type":"page","title":"Hello world","body":"初始页面"}}],"theme":"default","asideFixed":true,"asideFolded":false,"offScreen":false,"addPageIsOpen":false,"preview":false,"isMobile":false}
"""
from .models import AmisRenderList
import os
from django.conf import settings
import json


def update_amis_local_to_editor_one_file(route_id):
    base_dir = settings.BASE_DIR

    if settings.DEFAULT_AMIS_JSON_DIR is not None:
        #return HttpResponse('需要设置json本地文件目录', status=400)
        json_dir = settings.DEFAULT_AMIS_JSON_DIR
    else:
        json_dir = 'static/django_amis_render/json'

    pages=[]
    all_dir = os.path.join(base_dir, json_dir)

    ajf = AmisRenderList.objects.get(id=route_id)
    file_full_path=os.path.join(all_dir, ajf.file_path)
    if os.path.isfile(file_full_path):
        f = open(file_full_path, 'r', encoding='utf-8')
        data = f.read()
        f.close()
    else:
        #file not exist
        data = "{}"
    pages.append({'icon': '', 'id': str(1), 'label': ajf.file_name, 'path': ajf.file_name.replace('.json', ''), 'schema': json.loads(data)})

    store_data = {'pages':pages}
    store_data['addPageIsOpen']=False
    store_data['asideFixed'] = True
    store_data['isMobile'] = False
    store_data['offScreen'] = False
    store_data['theme'] = 'default'
    store_data['asideFolded'] = False


    return HttpResponse(json.dumps({'data':store_data,   }), status=200)

def update_amis_local_to_editor(request):
    route_id = request.POST.get('route_id')

    if route_id is not None:
        return update_amis_local_to_editor_one_file(route_id)

    return HttpResponse(json.dumps({'data':'ok',   }), status=200)

def update_amis_editor_to_local(request):

    if settings.DEFAULT_AMIS_JSON_DIR is not None:
        #return HttpResponse('需要设置json本地文件目录', status=400)
        json_dir = settings.DEFAULT_AMIS_JSON_DIR
    else:
        json_dir = 'static/django_amis_render/json'


    base_dir = settings.BASE_DIR
    all_dir = os.path.join(base_dir, json_dir)


    store = request.POST.get('store')
    print(store)
    #store=store.encode('utf-8').decode('raw_unicode_escape') #.decode('unicode_escape')

    pages = json.loads(store)
    pages=pages['pages']
    for i in pages:
        file_name = i['label']
        full_file_name = os.path.join(all_dir, file_name)
        print('saving file:', full_file_name)
        f = open(full_file_name, 'w', encoding='utf-8')
        f.write(json.dumps(i['schema'], ensure_ascii=False))#.encode('utf-8')
        f.close()

    return HttpResponse('ok', status=200)




# coding:utf-8

__author__ = "songjiangshan"
__copyright__ = "Copyright (C) 2021 songjiangshan \n All Rights Reserved."
__license__ = ""
__version__ = "1.0"

from django.db.models import Q
from rest_framework import viewsets
from rest_framework import serializers
from rest_framework.response import Response
from rest_framework.decorators import api_view
from django.views.decorators.csrf import csrf_exempt
import json
from django.http import HttpResponse
from .models import RouteExec
from django.shortcuts import render
from AmisBack.models import RouteExec, AmisJsonFile, AmisCfg
from .my_rest_api import my_rest_viewsetB
from .update_models import parse_params

s_custom_init_rest = 0

from django.urls import get_urlconf

def no_route_found(request):
    from amis_django.urls import urlpatterns
    nof = HttpResponse()
    nof.content = 'no page found:'+request.path
    nof.status_code = 404
    url_confs = get_urlconf()
    print(url_confs)

    return nof

def render_amis_json(request, amis_json):
    json_file_name = amis_json.file_name
    file_url = '/json/'+json_file_name
    page_name = amis_json.render_html
    print('render_amis_json', file_url)
    if (page_name is None) or len(page_name) == 0:
        page_name = 'html/index_for_json.html'
    return render(request, page_name, {'json_to_render_file_url': file_url, 'module_name': request.path})

def render_this_one(request, re_one):
    if re_one.re_type == 'py':
        pass  # python
    elif re_one.re_type == 'sql':
        pass  # sql
    elif re_one.re_type == 'json':
        pass  # json file
        return render(request, 'json/' + re_one.page_name)
    else:
        # unknown type
        nof = HttpResponse()
        nof.content = 'unknown type:' + str(re_one.re_type)
        nof.status_code = 400
        return nof


import os

from django.conf import settings
import platform
g_sys_name = platform.system()
#Windows Linux
def load_default_file(request):
    BASE_DIR = settings.BASE_DIR
    path = request.path
    global g_sys_name
    print(path)
    if path=='/':
        #no root index page
        return no_route_found(request)

    if len(path)<2:
        return no_route_found(request)

    base_dir = str(BASE_DIR)
    #path = path.replace('/custom/', '')
    path = path[1:]
    path_all = os.path.join(os.path.join(base_dir , settings.DEFAULT_AMIS_FILE_DIR), path)
    if g_sys_name=='Windows':
        path_all = path_all.replace('/', '\\')
    print('load_default_file', path_all)

    if os.path.isfile(path_all):
        return render(request, path)

    return no_route_found(request)


def my_custom_view(request):
    path = request.path
    print(path)

    re_all = RouteExec.objects.all()
    for i in re_all:
        if i.route.strip() == path.strip():
            return render_this_one(request, i)

    # 渲染json文件
    for i in AmisJsonFile.objects.all():
        if i.route.strip() == path.strip():
            return render_amis_json(request, i)

    #其他文件
    return load_default_file(request)


def upate_route(request):
    return HttpResponse("ok")


# 所选题目
# choice_problems = my_rest_viewsetB(Problem, 'PromblemSetV', ['id', 'name_english', 'name', 'is_used'], filter_fields=('is_used',))


if __name__ == '__main__':
    pass



# Generated by Django 3.2.8 on 2021-10-29 03:42

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AmisRenderList',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('page_url', models.TextField(blank=True, null=True)),
                ('file_path', models.TextField(blank=True, null=True)),
                ('html_template', models.TextField(blank=True, null=True)),
                ('json_file_url', models.TextField(blank=True, null=True)),
                ('file_type', models.TextField(blank=True, help_text='supported file type: json or html', null=True)),
            ],
            options={
                'verbose_name_plural': 'AMIS-PAGE',
                'db_table': 'amis_render_list',
                'managed': True,
            },
        ),
    ]
